/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.taglib;

import java.util.Set;											// 6.4.3.4 (2016/03/11)
import java.io.IOException;
import java.io.ObjectInputStream;

import org.opengion.fukurou.util.Options;
import org.opengion.fukurou.util.StringUtil;
import org.opengion.fukurou.util.TagBuffer;
import org.opengion.fukurou.util.XHTMLTag;
import org.opengion.fukurou.util.ToString;						// 6.1.1.0 (2015/01/17)
import org.opengion.fukurou.util.ArraySet;						// 6.4.3.4 (2016/03/11)
import org.opengion.hayabusa.common.HybsSystem;
import org.opengion.hayabusa.common.HybsSystemException;

import static org.opengion.fukurou.util.StringUtil.nval;

/**
 * プルダウンメニューを作成するHTML拡張タグです。
 *
 * プルダウンメニュー内の選択肢は、selectタグの内容であるoptionタグ、
 * またはqueryOptionタグによって指定します。
 * optionタグがセットされない場合は、name属性の、コードリソース からoption文字列より、
 * プルダウンメニューの選択肢を作成します。
 * name 属性に ラベルリソース のキーを与えることで、 ロケールにあわせたリソースを使用して、
 * 画面に表示します。ロケールは、ユーザー情報の lang 属性を初期値で使用し、
 * セットされていない場合は、リクエスト情報のロケールから取得します。
 * language 属性で定義されている場合は、そちらを優先します。
 *
 * @og.formSample
 * ●形式：&lt;og:select name="…" /&gt;
 * ●body：あり(EVAL_BODY_INCLUDE:BODYをインクルードし、{&#064;XXXX} は解析しません)
 *
 * ●Tag定義：
 *   &lt;og:select
 *       name             ○【TAG】メニューの名称を指定します(必須)。
 *       value              【TAG】値を指定します
 *       defaultVal         【TAG】value属性に値がセットされていないとき使用する、初期値を指定します
 *       lbl                【TAG】ラベルリソースのラベルIDを指定します
 *       must               【TAG】必須入力を表す色に変えるかどうか[true/false]を指定します(初期値:false)
 *       mustAny            【TAG】選択必須入力(どれかひとつ必須)を表す色[true/mustAny/その他]を指定します(初期値:無指定)
 *       useMustHidden      【TAG】必須の自動チェック用Hiddenを出力するかどうか[true/false]を指定します(初期値:true)
 *       addNoValue         【TAG】値なしのOptionを含めるかどうか[true/false]を指定します(初期値:false)
 *       addKeyLabel        【TAG】セレクトメニューの場合、キー:ラベル形式で表示するかどうか[true/false/null]を指定します(初期値:null)
 *       td                 【TAG】テーブル形式の &lt;td&gt; タグを使用するかどうか[yes/no/false]を指定します(初期値:yes)
 *       colspan            【TAG】フィールド部分の colspan を指定します
 *       rowspan            【TAG】ラベル、フィールド共に rowspan を指定します
 *       optionAttributes   【TAG】JavaScript などの HTML基本タグ以外の属性を、そのままタグとして使用します
 *       size               【TAG】リストボックスとして表示する場合の表示行数を指定します
 *       multiple           【TAG】複数選択を可能(multiple)にします(初期値:未設定)
 *       id                 【HTML】要素に対して固有の名前(id)をつける場合に設定します
 *       caseKey            【TAG】このタグ自体を利用するかどうかの条件キーを指定します(初期値:null)
 *       caseVal            【TAG】このタグ自体を利用するかどうかの条件値を指定します(初期値:null)
 *       caseNN             【TAG】指定の値が、null/ゼロ文字列 でない場合(Not Null=NN)は、このタグは使用されます(初期値:判定しない)
 *       caseNull           【TAG】指定の値が、null/ゼロ文字列 の場合は、このタグは使用されます(初期値:判定しない)
 *       caseIf             【TAG】指定の値が、true/TRUE文字列の場合は、このタグは使用されます(初期値:判定しない)
 *       lang               【HTML】要素の内容と他の属性値の言語(lang,xml:lang)を指定します
 *       dir                【HTML】文字表記の方向(dir)を指定します
 *       title              【HTML】要素に対する補足的情報(title)を設定します
 *       style              【HTML】この要素に対して適用させるスタイルシート(style)を設定します
 *       disabled           【TAG】その部品に対して、選択や変更が出来ないように(disabled)指定します(サーバーに送信されない)
 *       tabindex           【HTML】タブの移動順(tabindex)を指定します(0 ～ 32767)
 *       clazz              【HTML】要素に対して class 属性を設定します
 *       language           【TAG】タグ内部で使用する言語コード[ja/en/zh/…]を指定します
 *       onClick            【HTML】JavaScriptのイベント onClick を設定します(例:onClick="renew('query.jsp','QUERY');")
 *       onChange           【HTML】JavaScriptのイベント onChange を設定します(例:onChange="renew('query.jsp','QUERY');")
 *       onBlur             【HTML】JavaScriptのイベント onBlur を設定します(例:onBlur="this.value=value.toUpperCase();")
 *       onFocus            【HTML】JavaScriptのイベント onFocus を設定します
 *       ondblClick         【HTML】JavaScriptのイベント ondblClick を設定します
 *       onMouseDown        【HTML】JavaScriptのイベント onMouseDown を設定します
 *       onMouseUp          【HTML】JavaScriptのイベント onMouseUp を設定します
 *       onMouseMove        【HTML】JavaScriptのイベント onMouseMove を設定します
 *       onMouseOut         【HTML】JavaScriptのイベント onMouseOut を設定します
 *       onMouseOver        【HTML】JavaScriptのイベント onMouseOver を設定します
 *       onSelect           【HTML】JavaScriptのイベント onSelect を設定します
 *       onKeydown          【HTML】JavaScriptのイベント onKeydown を設定します
 *       onKeypress         【HTML】JavaScriptのイベント onKeypress を設定します
 *       onKeyup            【HTML】JavaScriptのイベント onKeyup を設定します
 *       autofocus          【HTML5】指定した入力欄にカーソルが当たって自動的にフォーカスされます。
 *       roles              【TAG】ロールをセットします
 *       eventColumn        【TAG】イベントカラム(親カラム)を指定します
 *       eventValue         【TAG】イベントValueのＳＱＬ分
 *       eventURL           【TAG】イベントカラム指定時に呼び出すURL
 *       eventCallback      【TAG】eventColumn実行後のcallbak関数指定
 *       debug              【TAG】デバッグ情報を出力するかどうか[true/false]を指定します(初期値:false)
 *   &gt;   ... Body ...
 *   &lt;/og:select&gt;
 *
 * ●使用例
 *    &lt;og:select name="CDC"&gt;                       CodeResource プロパティの値を使用したいときはcode属性を使います。
 *        &lt;og:option code="CDC" /&gt;
 *    &lt;/og:select&gt;
 *
 *    &lt;og:select name="CDC"&gt;                       選択項目の一番上に項目を付け足すことも可能です。
 *        &lt;og:option lbl="MSG0001" /&gt;              MessageResource プロパティの値を使用したいとききはlbl属性を使います。
 *        &lt;og:option code="CDC" /&gt;
 *    &lt;/og:select&gt;
 *
 *    &lt;og:select name="CDC"&gt;                       選択項目の一番上に項目を付け足すことも可能です。
 *        &lt;og:option lbl="CDC1" /&gt;                 LabelResource プロパティの値を使用したいとききはlbl属性を使います。
 *        &lt;og:option code="CDC" /&gt;
 *    &lt;/og:select&gt;
 *
 *    &lt;og:select name="CDC" multiple="multiple"&gt;   選択肢の中から複数選択できるようにするときはmultiple属性を使います。
 *        &lt;og:option code="CDC" /&gt;
 *    &lt;/og:select&gt;
 *
 *    &lt;og:select name="CDC" disabled="disabled"&gt;   選択不可にするときはdisabled属性を使います。
 *        &lt;og:option code="CDC" /&gt;
 *    &lt;/og:select&gt;
 *
 *    &lt;og:select name="CDC"&gt;                       選択肢をSELECT文の結果から作成したいときはqueryOptionタグと組み合わせて使います。
 *        &lt;og:queryOption&gt;
 *                    select NOSYN,NOSYN,':',NMSYN from DB01 ORDER BY 1
 *        &lt;/og:queryOption&gt;
 *    &lt;/og:select&gt;
 *
 *    &lt;og:select name="CDC" eventColumn="OYA" &gt;    親子関係のプルダウンを作る場合
 *        &lt;og:queryOption&gt;
 *                    select NOSYN,NOSYN,':',NMSYN from DB01
 *                     where CLM = "{&#064;OYA}" ORDER BY 1
 *        &lt;/og:queryOption&gt;
 *    &lt;/og:select&gt;
 *
 * @og.group 選択データ制御
 *
 * @version  4.0
 * @author	 Kazuhiko Hasegawa
 * @since    JDK5.0,
 */
public class SelectTag extends HTMLTagSupport implements OptionAncestorIF {
	/** このプログラムのVERSION文字列を設定します。	{@value} */
	private static final String VERSION = "6.8.0.0 (2017/06/02)" ;
	private static final long serialVersionUID = 680020170602L ;

	// 6.4.3.4 (2016/03/11) String配列 から、Setに置き換えます。
	private static final Set<String> TD_SET = new ArraySet<>( "yes" , "no" , "false" );

	// 4.0.0 (2005/01/31) HTML_LABEL_SEPARATOR を boolean 変数として取得します。
	private final String CLM = HybsSystem.sysBool( "HTML_LABEL_SEPARATOR" ) ? ":" : "" ;

	// 4.0.0 (2005/01/31) USE_MULTI_KEY_SELECT を定義しておきます。
	private final boolean USE_MULTI_KEY_SELECT =  HybsSystem.sysBool( "USE_MULTI_KEY_SELECT" ) ;

	private transient Options option		= new Options();
	private String  tdFlag		= "yes";	// yes:<td> タグを使用 / false:フィールドのみ / no:ラベルとフィールド

	// 3.0.1.3 (2003/03/11) colspan 属性を追加。
	private String  colspan     = "";		// 6.1.0.0 (2014/12/26) refactoring
	private String  rowspan		= "";		// 6.2.0.0 (2015/02/27) rowspan 属性を追加。

	// 3.5.4.0 (2003/11/25) value ,defaultVal  属性を追加。
	private String  value       ;
	private String	defaultVal	;		// value がセットされていないときの初期値
	private boolean multipleAll	;		// 3.8.0.9 (2005/10/17) 複数選択可能時に全選択を設定する。
	private String eventColumn	;		// 4.3.6.0 (2009/04/01) イベントカラム
	private String	eventValue	;		// 6.3.4.0 (2015/08/01) eventValue 追加
	private String eventURL		;		// 5.1.9.0 (2010/08/01) 動的プルダウン不具合対応

	private String	rawParam	;		// 5.1.7.0 (2010/06/01) 動的プルダウン実装見直し

	// 5.1.9.0 (2010/08/01) 値なしのOptionをSelectに含めるかどうか 新規追加
	private boolean addNoValue  ;		// 互換性のため、初期値は、false になります。
	private String  addKeyLabel	;		// 6.0.4.0 (2014/11/28) キー:ラベル形式で表示するかどうか

	/**
	 * デフォルトコンストラクター
	 *
	 * @og.rev 6.4.2.0 (2016/01/29) PMD refactoring. Each class should declare at least one constructor.
	 */
	public SelectTag() { super(); }		// これも、自動的に呼ばれるが、空のメソッドを作成すると警告されるので、明示的にしておきます。

	/**
	 * Taglibの開始タグが見つかったときに処理する doStartTag() を オーバーライドします。
	 *
	 * @og.rev 5.0.2.0 (2009/11/01) 複数パラメーターの選択に対応
	 * @og.rev 5.1.7.0 (2010/06/01) command=RENEWの場合はdefaultValを無視する。
	 * @og.rev 5.1.9.0 (2010/08/01) addNoValue 対応
	 * @og.rev 5.2.2.0 (2010/11/01) caseKey 、caseVal 属性対応
	 * @og.rev 5.3.7.0 (2011/07/01) 5.1.7.0 (2010/06/01)の対応で判定条件が逆転しているバグを修正
	 * @og.rev 6.3.1.0 (2015/06/28) 個人リソースのUSE_ADD_KEY_LABELで初期値設定
	 *
	 * @return	後続処理の指示( EVAL_BODY_INCLUDE )
	 */
	@Override
	public int doStartTag() {
		// 5.2.2.0 (2010/11/01) caseKey 、caseVal 属性対応
		if( useTag() ) {
			// 6.3.1.0 (2015/06/28) 個人リソースのUSE_ADD_KEY_LABELで初期値設定
			if( addKeyLabel == null ) { addKeyLabel = nval( sys( "USE_ADD_KEY_LABEL" ) , null ); }		// 4:個人設定可

			if( value == null ) {
	//			value = nval( getRequestValue( get( "name" ) ),defaultVal );
				// 5.1.7.0 (2010/06/01) command=RENEWの場合はdefaultValを無視する。
				final String cmd = pageContext.getRequest().getParameter( "command" );
				if( "RENEW".equals( cmd ) ) {
					value = StringUtil.array2line( getRequestValues( get( "name" ) ), "|" );
				}
				else {
					value = nval( StringUtil.array2line( getRequestValues( get( "name" ) ), "|" ),defaultVal );
				}
			}

			// 5.1.9.0 (2010/08/01) addNoValue 対応:option を最初にセットする。
			if( addNoValue ) {
				option.add( "<option value=\"\" ></option>" );
			}

			return EVAL_BODY_INCLUDE ;	// Body インクルード( extends TagSupport 時)
		}
		return SKIP_BODY ;				// Body を評価しない
	}

	/**
	 * タグリブオブジェクトをリリースします。
	 * キャッシュされて再利用されるので、フィールドの初期設定を行います。
	 *
	 * @og.rev 2.0.0.4 (2002/09/27) カスタムタグの release() メソッドを、追加
	 * @og.rev 2.0.0.8 (2002/10/09) yes/no/false で指定するように変更
	 * @og.rev 3.0.1.3 (2003/03/11) colspan 属性を追加。
	 * @og.rev 3.1.1.2 (2003/04/04) Tomcat4.1 対応。release2() を doEndTag()で呼ぶ。
	 * @og.rev 3.5.4.0 (2003/11/25) value ,defaultVal  属性を追加。
	 * @og.rev 3.8.0.9 (2005/10/17) 複数選択可能時に全選択を設定する。
	 * @og.rev 4.1.2.1 (2008/03/13) must , mustAny 属性を自動化します。
	 * @og.rev 4.3.6.0 (2009/04/01) eventColumn,eventURL追加
	 * @og.rev 5.1.7.0 (2010/06/01) 動的プルダウン実装見直し
	 * @og.rev 5.1.9.0 (2010/08/01) columnタグで動的プルダウンが動作しない不具合に対応
	 * @og.rev 5.1.9.0 (2010/08/01) addNoValue  属性を追加。
	 * @og.rev 5.2.1.0 (2010/10/01) must , mustAny 属性の処理を、HTMLTagSupport に移します。
	 * @og.rev 6.0.4.0 (2014/11/28) addKeyLabel 属性を追加
	 * @og.rev 6.2.0.0 (2015/02/27) rowspan 属性を追加
	 * @og.rev 6.3.4.0 (2015/08/01) eventValue 追加
	 */
	@Override
	protected void release2() {
		super.release2();
		option		= new Options();
		tdFlag     = "yes";	// table形式の <td> タグを使用するかどうか
		colspan 	= "";		// 6.1.0.0 (2014/12/26) refactoring
		rowspan		= "";		// 6.2.0.0 (2015/02/27) rowspan 属性を追加
		value       = null;
		defaultVal	= null;		// value がセットされていないときの初期値
		multipleAll	= false;	// 3.8.0.9 (2005/10/17) 複数選択可能時に全選択を設定する。
		eventColumn	= null;		// 4.3.6.0 (2009/04/01)
		eventValue  = null;  	// 6.3.4.0 (2015/08/01)
		eventURL	= null;		// 5.1.9.0 (2010/08/01) columnタグで動的プルダウンが動作しない不具合に対応
		rawParam	= null;		// 5.1.7.0 (2010/06/01) 動的プルダウン実装見直し
		addNoValue  = false;	// 5.1.9.0 (2010/08/01) 互換性のため、初期値は、false になります。
		addKeyLabel = null;		// 6.0.4.0 (2014/11/28) キー:ラベル形式で表示するかどうか
	}

	/**
	 * 入力用のプルダウンメニューを作成します。
	 *
	 * 表示順の項目名タグは、あらかじめ作成済みの値をここでは受け取ります。
	 * hasSelect 属性の値に応じて，タグの作成方法は変わります。
	 *
	 * @og.rev 2.0.0.8 (2002/10/09) yes/no/false で指定するように変更
	 * @og.rev 2.0.1.0 (2002/10/10) ラベルとフィールドのセパレーターとして、コロン(：)を使用するかどうかを指定できる
	 * @og.rev 3.0.1.3 (2003/03/11) colspan 属性を追加。
	 * @og.rev 3.5.4.0 (2003/11/25) value ,defaultVal  属性を追加。
	 * @og.rev 4.0.0.0 (2005/01/31) マルチセレクションのON/OFFをここで指定。
	 * @og.rev 4.1.2.1 (2008/03/13) must , mustAny 属性を自動化します。
	 * @og.rev 4.3.6.0 (2009/04/01) eventColumn対応
	 * @og.rev 4.3.7.1 (2009/06/08) id=labelのclass化
	 * @og.rev 5.1.7.0 (2010/06/01) 動的プルダウン実装見直し
	 * @og.rev 5.2.1.0 (2010/10/01) must , mustAny 属性の処理を、HTMLTagSupport に移します。
	 * @og.rev 5.5.3.0 (2012/06/01) idのセット見直し
	 * @og.rev 5.6.2.2 (2013/03/15) 自動must処理の出力位置を変更
	 * @og.rev 6.2.0.0 (2015/02/27) rowspan 属性を追加。makeMustHidden(String)対応
	 * @og.rev 6.3.4.0 (2015/08/01) eventValue 追加
	 *
	 * @return	表示順選択用のプルダウンメニュータグ
	 * @og.rtnNotNull
	 */
	@Override
	protected String makeTag() {

		final String name = get( "name" );
		if( getMsglbl() == null ) { setLbl( name ); }

		// 6.1.0.0 (2014/12/26) refactoring

		// 4.0.0 (2005/01/31) XHTMLTag.java より移動)
		if( USE_MULTI_KEY_SELECT ) {
			set( "onkeydown", "setKeySelect(this);" );
		}

		// 5.5.3.0 既にIDが指定されている場合はnameをセットしない
		if( get("id")==null || get("id").isEmpty() ){
			set( "id", name );
		}

		// 4.3.6.0 (2009/04/01) eventColumnの対応
		// 5.1.7.0 (2010/06/01) 動的プルダウン実装見直し
		String selTag = XHTMLTag.select( getAttributes(), option );

		// 5.1.7.0 (2010/06/01) 動的プルダウン実装見直し
		if( eventColumn != null && eventColumn.length() > 0 ) {
			String editor = null;
			if( rawParam != null && rawParam.length() > 0 ) {
				editor = "DBMENU";
			}
			else {
				editor = "MENU";
			}
			addEventColumn( name, eventColumn, eventValue , eventURL, "", editor, "", rawParam );	// 6.3.4.0 (2015/08/01)

			// 6.1.1.0 (2015/01/17) TagBufferの連結記述
			selTag = new TagBuffer( "span" )
						.add( "class"							, HybsSystem.EVENT_COLUMN_CLASS )
						.add( HybsSystem.EVENT_COLUMN_ID		, name )
						.add( HybsSystem.EVENT_COLUMN_INITVAL	, value )
						.add( HybsSystem.EVENT_COLUMN_WRITABLE	, "true" )
						.addBody( selTag )
						.makeTag();

		}

		final StringBuilder rtn = new StringBuilder( BUFFER_MIDDLE );
		if( "yes".equals( tdFlag ) ) {
			// 4.3.7.1 (2009/06/08) id=labelのclass化
			// rtn.append( "<td id=\"label\">" );
			rtn.append( "<td class=\"label\"" )
				.append( rowspan ).append( '>' )		// 6.2.0.0 (2015/02/27)
				.append( getLongLabel() )				// 4.0.0 (2005/01/31)
				.append( CLM )
				.append( "</td><td" )
				.append( colspan )						// 6.1.0.0 (2014/12/26) refactoring
				.append( rowspan )						// 6.2.0.0 (2015/02/27)
				.append( '>' )							// 6.0.2.5 (2014/10/31) char を append する。
				.append( selTag )
				.append( makeMustHidden( name ) )		// 6.2.0.0 (2015/02/27) , 5.6.2.2 (2013/03/15)
				.append( "</td>" );
		}
		else if( "no".equals( tdFlag ) ) {
			// 4.3.7.1 (2009/06/08) id=labelのclass化
			// rtn.append( "<span id=\"label\">" );
			rtn.append( "<span class=\"label\">" )
				.append( getLongLabel() )		// 4.0.0 (2005/01/31)
				.append( CLM )
				.append( "</span>" )
				.append( selTag )
				.append( makeMustHidden( name ) );		// 6.2.0.0 (2015/02/27) , 5.6.2.2 (2013/03/15)
		}
		// "false".equals( tdFlag ) の条件
		else {
			rtn.append( selTag )
				.append( makeMustHidden( name ) );		// 6.2.0.0 (2015/02/27) , 5.6.2.2 (2013/03/15)
		}

		return rtn.toString() ;
	}

	/**
	 * メニュー項目の選択項目を追加します。
	 *
	 * select タグのBODY要素の OptionTag よりアクセスされます。
	 *
	 * @og.rev 3.1.2.0 (2003/04/07) taglib パッケージ内部で使用している箇所を protected 化する。
	 * @param	opt 	 オプションタグ文字列
	 * @see		org.opengion.hayabusa.taglib.OptionAncestorIF#addOption( String )
	 */
	public void addOption( final String opt ) {
		option.add( opt );
	}

	/**
	 * メニュー項目の最後の項目を削除します。
	 *
	 * select タグのBODY要素の OptionTag よりアクセスされます。
	 *
	 * @og.rev 6.8.0.0 (2017/06/02) メニュー項目の最後の項目を削除。
	 * @see		org.opengion.hayabusa.taglib.OptionAncestorIF#removeLast()
	 */
	public void removeLast() {
		option.removeLast();
	}

	/**
	 * 【TAG】メニューの名称を指定します。
	 *
	 * @og.tag メニューの名称を指定します。
	 *
	 * @param	name 名称
	 */
	public void setName( final String name ) {
		set( "name", getRequestParameter( name ) );
	}

	/**
	 * 【TAG】リストボックスとして表示する場合の表示行数を指定します。
	 *
	 * @og.tag 表示行数を指定します。
	 *
	 * @param	size 表示行数
	 */
	public void setSize( final String size ) {
		set( "size",getRequestParameter( size ) );
	}

	/**
	 * 【TAG】複数選択を可能(multiple)にします(初期値:未設定)。
	 *
	 * @og.tag
	 * multiple 値が "multiple" または "true" の場合、複数選択状態になります。
	 * また、"ALL" を設定する場合は、OptionTag上で、すべてのOption が選択状態に
	 * 初期設定されます。
	 * 大文字、小文字の種別はなくしますが、指定以外の文字が設定された場合は、エラーになります。
	 *
	 * @og.rev 3.8.0.9 (2005/10/17) 複数選択可能時に全選択を設定する。
	 *
	 * @param	multi 複数選択指定 [multiple:複数選択/true:複数選択/ALL:初期値全選択]
	 */
	public void setMultiple( final String multi ) {
		final String multiple = nval( getRequestParameter( multi ),null );
		if( "multiple".equalsIgnoreCase( multiple ) || "true".equalsIgnoreCase( multiple ) ) {
			set( "multiple","multiple" );
		}
		else if( "ALL".equalsIgnoreCase( multiple ) ) {
			set( "multiple","multiple" );
			multipleAll = true;
		}
		else if( ! ( "false".equalsIgnoreCase( multiple ) || multiple == null ) ) {
			final String errMsg = "multiple 属性には、[multiple,true,ALL] 以外は設定できません。"
						+ "multiple=" + multi ;
			throw new HybsSystemException( errMsg );
		}
	}

	/**
	 * 【TAG】テーブル形式の &lt;td&gt; タグを使用するかどうか[yes/no/false]を指定します(初期値:yes)。
	 *
	 * @og.tag
	 * 初期値は、使用する("yes") です。
	 *
	 * @og.rev 2.0.0.8 (2002/10/09) yes/no/false で指定するように変更
	 * @og.rev 5.5.1.0 (2012/04/03) エラーメッセージ変更
	 * @og.rev 6.3.4.0 (2015/08/01) Arrays.toString から String.join に置き換え。
	 * @og.rev 6.4.3.4 (2016/03/11) String配列 から、Setに置き換えます。
	 *
	 * @param   flag TDタグ使用 [yes:tdタグを使用/no:ラベルとフィールド/false:フィールドのみ]
	 */
	public void setTd( final String flag ) {
		final String td = nval( getRequestParameter( flag ),tdFlag );

		if( check( td , TD_SET ) ) {
			tdFlag = td;
		}
		else {
			final String errMsg = "Error in SelectTag [td]: "
						 + td + "  in [" + String.join( ", " , TD_SET ) + "] only used.";
			throw new HybsSystemException( errMsg );
		}
	}

	/**
	 * 【TAG】フィールド部分の colspan を指定します。
	 *
	 * @og.tag
	 * 通常は、ラベルとフィールドが、別のtdで囲われます。しかし、場合によっては、
	 * フィールド部の長いカラムがある場合、上下のカラムとの位置関係がおかしくなり、
	 * ラベルとフィールドが離れて表示されるケースがあります。
	 * この場合、長いフィールド部を持つカラムに、colspan 属性を指定すると、
	 * その分のカラムを割り当てることにより、上下のカラムとの位置関係を
	 * 調節することが可能になります。
	 * 通常は、3 を指定して、tdが、フィールド、ラベル、フィールドの
	 * 組み合わせ部分に、長いフィールドが適用されるように使用します。
	 *
	 *  &lt;og:select name="CDK" colspan="3" /&gt;
	 *
	 * @og.rev 3.0.1.3 (2003/03/11) colspan 属性を追加。
	 *
	 * @param   sp フィールド部分のcolspan
	 */
	public void setColspan( final String sp ) {
		colspan = nval( getRequestParameter( sp ),colspan );
		// 6.1.0.0 (2014/12/26) refactoring
		if( colspan.length() > 0 ) {
			colspan = " colspan=\"" + colspan + "\" ";
		}
	}

	/**
	 * 【TAG】ラベル、フィールド共に rowspan を指定します。
	 *
	 * @og.tag
	 * 通常は、ラベルとフィールドが、別のtdで囲われます。rowspan は、
	 * そのどちらの TDフィールドにも、設定します。
	 * 初期値は、rowspan を付けないです。
	 *
	 *  &lt;og:select name="CDK" rowspan="3" /&gt;
	 *
	 * @og.rev 6.2.0.0 (2015/02/27) rowspan 属性を追加
	 *
	 * @param   sp ラベル、フィールド部分のrowspan
	 */
	public void setRowspan( final String sp ) {
		rowspan = nval( getRequestParameter( sp ),rowspan );
		if( rowspan.length() > 0 ) {
			rowspan = " rowspan=\"" + rowspan + "\" ";
		}
	}

	/**
	 * 【TAG】値を指定します。
	 *
	 * @og.tag
	 *   ここで指定された値が、optionタグの値と同じであれば、選択状態になります。
	 *
	 *    &lt;og:select name="CDC" value="{&#064;ABC}" defaultVal="XYZ"  &gt;
	 *         &lt;og:option value="ABC" lbl="CDC1" /&gt;
	 *         &lt;og:option value="XYZ" lbl="CDC2" /&gt;
	 *    &lt;/og:select&gt;
	 *
	 * @og.rev 3.5.4.0 (2003/11/25) 新規作成
	 * @og.rev 5.0.2.0 (2009/11/01) 複数パラメーターの選択に対応
	 *
	 * @param   val 値
	 */
	public void setValue( final String val ) {
		value = nval( StringUtil.array2line( getRequestParameterValues( val ), "|" ),value );
	}

	/**
	 * 値を外部から取り出します。
	 *
	 * OptionTag で、value を取り出して、内部の値と同じ場合は、選択状態にします。
	 *
	 * @og.rev 3.5.4.0 (2003/11/25) 新規作成
	 * @og.rev 5.7.1.0 (2013/12/06) OptionAncestorIF にする為、protected ⇒ public 化する。
	 *
	 * @return	内部に設定された値
	 * @see		org.opengion.hayabusa.taglib.OptionAncestorIF#getValue()
	 */
	public String getValue() {
		return value;
	}

	/**
	 * 【TAG】イベントカラム(親カラム)を設定します。
	 *
	 * @og.tag
	 *   親子関係のカラムを作成する場合に利用します。
	 *   queryOptionタグとセットで利用して下さい。
	 *
	 *   ※詳細はcolumnTagのeventColumn属性の説明をご覧下さい。
	 *
	 * @og.rev 4.3.6.0 (2009/04/01)
	 *
	 * @param   col 親カラム名
	 */
	public void setEventColumn( final String col ) {
		eventColumn = nval( getRequestParameter( col ), eventColumn );
	}

	/**
	 * 【TAG】eventColumn使用時の値を、SQL文から求めるためのqueryをセットします。
	 *
	 * @og.tag
	 * eventColumn利用時は{&#064;XXXX}はリクエストパラメータではなく
	 * xxxカラムの実際の値が利用されます。
	 *
	 * @og.rev 6.3.4.0 (2015/08/01) eventValue 追加
	 * @param	val	パラメータ
	 */
	public void setEventValue( final String val ) {
		eventValue = nval( getReservedParameter( val ), eventValue );	// ここでは{@XXXX}変換をしない
	}

	/**
	 * 【TAG】イベントカラム指定時に呼び出すURLを指定します。
	 *
	 * @og.tag
	 *   イベントカラム指定時に部品を作成するJSPを指定します。
	 *   初期値はシステムリソースのEVENT_COLUMN_URLです。
	 *   (例：eventURL="makeColumn_custom.jsp")
	 *
	 *   ※詳細はcolumnTagのeventURL属性の説明をご覧下さい。
	 *
	 * @og.rev 4.3.6.0 (2009/04/01)
	 *
	 * @param	url	呼び出すURL
	 */
	public void setEventURL( final String url ) {
		eventURL = nval( getRequestParameter( url ), eventURL );
	}

	/**
	 * 複数選択可能時に全選択を設定するかどうかを返します。
	 *
	 * これは、上位入れ子のタグの OptionTag で、multipleAll を取り出して、
	 * true であれば、全選択に設定します。
	 *
	 * @og.rev 3.8.0.9 (2005/10/17) 新規作成
	 * @og.rev 5.7.1.0 (2013/12/06) OptionAncestorIF にする為、protected ⇒ public 化する。
	 *
	 * @return	全選択:true / 通常:false
	 * @see		org.opengion.hayabusa.taglib.OptionAncestorIF#isMultipleAll()
	 */
	public boolean isMultipleAll() {
		return multipleAll;
	}

	/**
	 * 【TAG】value属性に値がセットされていないとき使用する、初期値を指定します。
	 *
	 * @og.tag
	 * value属性に値がセットされていないときに、この初期値を使用します。
	 *
	 * なお、commandがRENEWの場合は、このdefaultValは適用されません。
	 * これは、defaultValの値が埋め込まれた項目が存在する画面に戻った際に、
	 * defaultValの項目の値がnullとなっていた場合に、defaultValの値が復元されると、
	 * 検索結果との不一致が発生するためです。
	 *
	 * @og.rev 3.5.4.0 (2003/11/25) 新規作成
	 * @og.rev 5.0.2.0 (2009/11/01) 複数パラメーターの選択に対応
	 *
	 * @param	dv 初期値
	 */
	public void setDefaultVal( final String dv ) {
		defaultVal = nval( StringUtil.array2line( getRequestParameterValues( dv ), "|" ),defaultVal );
	}

	/**
	 * 【TAG】値なしのOptionを含めるかどうか[true/false]を指定します(初期値:false)。
	 *
	 * @og.tag
	 * カラムがプルダウンメニュー形式の場合、選択肢には通常データベース登録できる候補が
	 * 並びます。しかし、検索時には、候補を絞りたくない(指定しない)場合があり、
	 * その様な場合には、値なしのOptionを含める(true)ことで、対応可能です。
	 * 初期値:false(値なしのOptionを含めない)は、過去の互換性を重視しているため、column タグと異なります。
	 *
	 * @og.rev 5.1.9.0 (2010/08/01) 新規追加
	 *
	 * @param	flag  値なしのOptionの有無 [true:含める/それ以外:含めない]
	 */
	public void setAddNoValue( final String flag ) {
		addNoValue = nval( getRequestParameter( flag ),addNoValue );
	}

	/**
	 * 【TAG】セレクトメニューの場合、キー:ラベル形式で表示するかどうか[true/false/null]を指定します(初期値:null)。
	 *
	 * @og.tag
	 * カラムがプルダウンメニュー形式の場合、キー:ラベル形式で表示するか、ラベルだけで表示するかを
	 * 選択できます。
	 * true を指定すると、「強制的に」キー:ラベル形式で表示します。
	 * false の場合は、「強制的に」ラベルのみで表示されます。
	 * 初期値の null の場合、コードリソースや、SQL文に準拠した指定通りの形式で表示します。
	 *
	 * &lt;og:column name="CDC" addKeyLabel="true" /&gt;
	 *
	 * @og.rev 6.0.4.0 (2014/11/28) キー:ラベル形式で表示するかどうか。新規追加
	 *
	 * @param   flag  キー:ラベル形式表示 [true:キー:ラベル形式/false:ラベルのみ/null:指定通り]
	 */
	public void setAddKeyLabel( final String flag ) {
		addKeyLabel = nval( getRequestParameter( flag ),addKeyLabel );
	}

	/**
	 * セレクトメニューの場合、キー:ラベル形式で表示するかどうか[true/false/null]を返します。
	 *
	 * これは、上位入れ子のタグの OptionTag で、addKeyLabel を取り出して、
	 * true であれば、キー:ラベル形式 のオプションを、#addOption( String ) で
	 * 登録させます。
	 *
	 * @og.rev 6.0.4.0 (2014/11/28) キー:ラベル形式で表示するかどうか。新規追加
	 *
	 * @return	true:キー:ラベル形式/false:ラベルのみ/null:指定通り
	 * @see		#addOption( String )
	 * @see		org.opengion.hayabusa.taglib.OptionAncestorIF#getAddKeyLabel()
	 */
	public String getAddKeyLabel() {
		return addKeyLabel;
	}

	/**
	 * パラメーター変換({&#064;XXXX}の置き換えをしない状態のパラメーターをセットします。
	 *
	 * @og.rev 5.1.7.0 (2010/06/01) 新規作成(動的プルダウン実装見直し)
	 * @og.rev 5.5.4.0 (2012/07/02) 予約語対応
	 *
	 * @param   param パラメーター
	 * @see		org.opengion.hayabusa.taglib.OptionAncestorIF#setRawParam( String )
	 */
	public void setRawParam( final String param ) {
		rawParam = nval( getReservedParameter(param), rawParam ); // 5.5.4.0 (2012/07/02)
	}

	/**
	 * 【TAG】eventColumn実行後のcallbak関数を指定します。
	 *
	 * @og.tag
	 * eventColumnの動作終了後に実行するCallback関数の指定を行います。
	 * 関数名のみを指定して下さい。
	 * 　ex) sampleCallback()を実行する場合はeventCallback="sampleCallback"
	 * 実行する関数の第一引数には自動的に次の要素が入った配列が渡されます
	 * 　[0] 変更された子カラムのID属性値
	 * 　[1] evnetColumn実行後の子カラムの値
	 * 　[2] eventColumn実行前の子カラムの値
	 *
	 * この属性は、optionAttributesへの設定と同様の動作を行います。
	 *
	 * @og.rev 5.5.4.0 (2012/07/02) 新規追加
	 *
	 * @param   callback callbak関数
	 */
	public void setEventCallback( final String callback ) {
		// optionAttributes扱いで登録します。
		if( callback != null && callback.length() > 0 ){
			add( "optionAttributes", "eventCallback='"+getRequestParameter( callback )+"'" );
		}
	}

	/**
	 * シリアライズ用のカスタムシリアライズ読み込みメソッド
	 *
	 * ここでは、transient 宣言された内部変数の内、初期化が必要なフィールドのみ設定します。
	 *
	 * @og.rev 4.0.0.0 (2006/09/31) 新規追加
	 * @serialData 一部のオブジェクトは、シリアライズされません。
	 *
	 * @param	strm	ObjectInputStreamオブジェクト
	 * @see #release2()
	 * @throws IOException	シリアライズに関する入出力エラーが発生した場合
	 * @throws ClassNotFoundException	クラスを見つけることができなかった場合
	 */
	private void readObject( final ObjectInputStream strm ) throws IOException , ClassNotFoundException {
		strm.defaultReadObject();
		option = new Options();
	}

	/**
	 * このオブジェクトの文字列表現を返します。
	 * 基本的にデバッグ目的に使用します。
	 *
	 * @return このクラスの文字列表現
	 * @og.rtnNotNull
	 */
	@Override
	public String toString() {
		return ToString.title( this.getClass().getName() )
				.println( "VERSION"		,VERSION		)
				.println( "tdFlag"		,tdFlag		)
				.println( "colspan"		,colspan 		)
				.println( "rowspan"		,rowspan		)
				.println( "value"		,value			)
				.println( "defaultVal"	,defaultVal		)
				.println( "multipleAll"	,multipleAll	)
				.println( "Other..."	,getAttributes().getAttribute() )
				.fixForm().toString() ;
	}
}
